package ssw.mj;

/**
 * A <code>Token</code> represents a terminal symbol.
 * Tokens are provided by the scanner for the parser.
 * They hold additional information about the symbol.
 *
 * @author AW
 */
public class Token {

	/*----- Token codes */
	public static final int
		none       =  0,
		ident      =  1,
		number     =  2,
		charConst  =  3,
		plus       =  4,  /* + */
		minus      =  5,  /* - */
		times      =  6,  /* * */
		slash      =  7,  /* / */
		rem        =  8,  /* % */
		eql        =  9,  /* == */
		neq        = 10,  /* != */
		lss        = 11,  /* < */
		leq        = 12,  /* <= */
		gtr        = 13,  /* > */
		geq        = 14,  /* >= */
		and        = 15,  /* && */
		or         = 16,  /* || */
		assign     = 17,  /* = */
		plusas     = 18,  /* += */
		minusas    = 19,  /* -= */
		timesas    = 20,  /* *= */
		slashas    = 21,  /* /= */
		remas      = 22,  /* %= */
		pplus      = 23,  /* ++ */
		mminus     = 24,  /* -- */
		semicolon  = 25,  /* ; */
		comma      = 26,  /* , */
		period     = 27,  /* . */
		lpar       = 28,  /* ( */
		rpar       = 29,  /* ) */
		lbrack     = 30,  /* [ */
		rbrack     = 31,  /* ] */
		lbrace     = 32,  /* { */
		rbrace     = 33,  /* } */
		break_     = 34,
		class_     = 35,
		else_      = 36,
		final_     = 37,
		if_        = 38,
		new_       = 39,
		print      = 40,
		program    = 41,
		read       = 42,
		return_    = 43,
		void_      = 44,
		while_     = 45,
		eof        = 46;

	/** Token class (none, ident, ...). */
	int kind;

	/** Line number of this token. */
	int line;

	/** Column number of this token. */
	int col;

	/** Value of this token (for numbers or character constants). */
	int val;

	/** String representation of this token. */
	String str;

	public Token (int kind, int line, int col) {
		this.kind = kind; this.line = line; this.col = col;
	}

	/**
	 * Returns the readable name of a given token kind.
	 */
	public static String kindToString(int kind) {
		return Messages.getString("Token." + kind);
	}
	
	/**
	 * Returns a string representation of this Token object.
	 * This only includes the relevant attributes of the token.
	 */
	@Override
	public String toString () {
		String result = "line " + line + ", col " + col + ", kind " + kind + " (" + kindToString(kind) + ")";
		if (kind == ident) {
			result = result + ", str " + str;
		} else if (kind == number) {
			result = result + ", val " + val;
		} else if (kind == charConst) {
			result = result + ", val '" + (char) val + "'";
		}
		return result;
	}
}
